const Util = require('./util');
const request = require('request');



module.exports  = function(config){

    const requester = {};

    requester.list = function () {

        const promise = new Promise(function (resolve, reject) {

            request.get({
                url: config.INFOCHAT_DATA_SERVER + '/chattrees',
                json: true,
                headers: {
                    'Authorization': config.USER_TOKEN
                }
            }, function (err, httpResponse, body) {

                if (!err) {

                    console.log('######################');
                    //console.log(body);
                    resolve(body);
                } else {
                    console.log('erro ao listar dados', err);
                    reject(err);
                }

            });

        });

        return promise;
    };


    requester.listCompanies = function () {

        const promise = new Promise(function (resolve, reject) {


            request.get({
                url: config.INFOCHAT_DATA_SERVER + '/companies',
                json: true,
                headers: {
                    'Authorization': config.USER_TOKEN
                }
            }, function (err, httpResponse, body) {

                if (!err) {
                    resolve(body);
                } else {
                    console.log('erro ao listar dados', err);
                    reject(err);
                }

            });

        });


        return promise;
    };


    requester.findCompany = function (id) {

        const promise = new Promise(function (resolve, reject) {

            request.get({
                url: config.INFOCHAT_DATA_SERVER + '/companies/'+id,
                json: true,
            }, function (err, httpResponse, body) {

                if (!err) {
                    resolve(body);
                } else {
                    console.log('erro ao listar dados', err);
                    reject(err);
                }

            });

        });


        return promise;
    };


    requester.findTree = function (id) {

        const promise = new Promise(function (resolve, reject) {

            request.get({
                url: config.INFOCHAT_DATA_SERVER + '/chattrees/'+id,
                json: true,
                headers: {
                    'Authorization': config.USER_TOKEN
                }
            }, function (err, httpResponse, body) {

                if (!err) {

                    try{
                        resolve(body);
                    }catch (e){
                        console.log(e);
                        reject(e);
                    }


                } else {
                    console.log('erro ao listar dados', err);
                    reject(err);
                }

            });

        });


        return promise;
    };


    requester.findTreeByCompanyId = function (treeId) {

        const promise = new Promise(function (resolve, reject) {

            request.get({
                url: config.INFOCHAT_DATA_SERVER + '/chattrees/'+treeId+'/findByCompanyId',
                json: true,
                headers: {
                    'Authorization': config.USER_TOKEN
                }
            }, function (err, httpResponse, body) {

                if (!err) {
                    resolve(body);
                } else {
                    console.log('erro ao listar dados', err);
                    reject(err);
                }

            });

        });


        return promise;
    };


    requester.listKeywordsExpressions = function () {

        return new Promise(function (resolve, reject) {
            request.get({
                url: config.INFOCHAT_DATA_SERVER + '/keywordintents',
                json: true,
            }, function (err, httpResponse, body) {

                if (!err) {

                    try {
                        resolve(body);
                    } catch (e) {
                        console.log(e);
                        reject(e);
                    }

                } else {
                    console.log('erro ao listar dados', err);
                    reject(err);
                }

            });

        });

    };


    requester.findBookingEngineByType = function (type) {

        const promise = new Promise(function (resolve, reject) {

            request.get({
                url: config.INFOCHAT_DATA_SERVER + '/bookingengines/findByType/'+type,
                json: true,
                headers: {
                    'Authorization': config.USER_TOKEN
                }
            }, function (err, httpResponse, body) {

                if (!err) {
                    resolve(body);
                } else {
                    console.log('erro ao listar dados', err);
                    reject(err);
                }

            });

        });


        return promise;
    };


    requester.finishConversation = function (conversationId) {

        const promise = new Promise(function (resolve, reject) {

            request.get({
                url: config.INFOCHAT_DATA_SERVER + '/conversation/finishConversation/'+conversationId,
                headers: {
                    'Authorization': config.USER_TOKEN
                }
            }, function (err, httpResponse, body) {

                if (!err) {

                    console.log('finishedconversation', body);
                    resolve(body);
                } else {
                    console.log('erro ao realizar o finish', err);
                    reject(err);
                }

            });

        });

        return promise;
    };

    requester.markRequestPrice = function (conversationId) {
        return new Promise(function (resolve, reject) {
            request.post({
                url: config.INFOCHAT_DATA_SERVER + '/conversation/setRequestedPrice/' + conversationId,
                headers: {
                    'Authorization': config.USER_TOKEN
                }
            }, function (err, httpResponse, body) {

                if (!err && httpResponse.statusCode < 400) {
                    console.log('mark conversation request price', conversationId);
                    resolve(body);
                } else {
                    console.log('erro ao realizar o mark conversation request price', err || body);
                    reject(err);
                }

            });

        });
    };

    requester.registerQuote = function (searchData, isTest = false) {
        return new Promise(function (resolve, reject) {
            searchData.companyId = searchData.companyId.split('.')[1];

            if(isTest) {
                resolve('test chat');
            } else {

                const [arrivalDate, arrivalTime] = Util.getDateAndTimeFromDateISO(searchData.arrivalDate);
                const [departureDate, departureTime] = Util.getDateAndTimeFromDateISO(searchData.departureDate);

                searchData.arrivalTime = arrivalTime;
                searchData.arrivalDate = Util.appendTimeISOStringToDate(arrivalDate, '06');

                searchData.departureTime = departureTime;
                searchData.departureDate =  Util.appendTimeISOStringToDate(departureDate, '06');

                if (searchData.searchResult) {
                    searchData = Object.assign(searchData, Util.getPriceExtremes(searchData.searchResult));
                }

                request.post({
                    url: config.INFOCHAT_DATA_SERVER + '/conversation/registerQuote',
                    headers: {
                        'Authorization': config.USER_TOKEN
                    },
                    json: searchData
                }, function (err, httpResponse, body) {

                    if (!err) {
                        resolve(body);
                    } else {
                        reject(err);
                    }

                });
            }
        });
    };


    return requester;
};




